#include <RunningMedian.h>


// GPIOs
#define XFADELEFT     39
#define XFADERIGHT    36
#define XFADECURVE    35

float xfadecurve;


// Potentiometer input stuff
#define SRC1_LEFT     0
#define SRC1_RGHT     1
#define SRC2_LEFT     2
#define SRC2_RGHT     3

#define HI_DEADZN     0
#define LO_DEAZN      0

#define MAXPOSITION   4095
#define MIDPOSITION   (MAXPOSITION*0.6)
#define MIDPOSOVERLAP (MAXPOSITION*0.2)
#define LOWPOSITION   (MAXPOSITION*0.05)
#define MINPOSITION   0
#define KNEEPOS       0.55


#define MAXOUT        1000                // Full volume
#define NORMALOUT     (MAXOUT*0.98)
#define BOOST         (MAXOUT-NORMALOUT)
#define LOWOUT        (NORMALOUT*0.75)    // Volume when the joystick is at a specific position (LOWPOSITION) on the lower half of the movenent
#define MINOUT        0                   // Silent



#define ARRAYSIZE 3
RunningMedian avgarray[4] = { RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE) }; 





// ------------------------------------------------------------------------------
void readfaders(void) {

  xfadecurve = 1 - analogRead(XFADECURVE)/(float)MAXPOSITION;    // Adjustable volume for the centered stick deadzone

  // read positions and add to averaging array
  avgarray[SRC1_LEFT].add(map(constrain(analogRead(XFADELEFT),  LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MAXPOSITION, MINPOSITION));
  avgarray[SRC1_RGHT].add(map(constrain(analogRead(XFADERIGHT), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MAXPOSITION, MINPOSITION));
  avgarray[SRC2_LEFT].add(map(constrain(analogRead(XFADELEFT),  LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
  avgarray[SRC2_RGHT].add(map(constrain(analogRead(XFADERIGHT), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
}



// ------------------------------------------------------------------------------
float xfade (uint8_t channel, uint8_t opposite) {

  int result;
  int stickPos = avgarray[channel].getMedian();
  int oppPos = avgarray[opposite].getMedian();
  
//  float x = (stickPos-oppPos)/(float)MAXPOSITION;
//  float y = ((stickPos+oppPos)/(float)MAXPOSITION)-1;

  float tophalf      = map(constrain((stickPos+oppPos)/2, KNEEPOS*MAXPOSITION, MAXPOSITION), KNEEPOS*MAXPOSITION, MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  float firstquarter = map(constrain((stickPos+oppPos)/2, 0.5*MAXPOSITION, 0.75*MAXPOSITION), 0.5*MAXPOSITION, 0.75*MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  int midposition = MIDPOSITION-MIDPOSOVERLAP + ( MIDPOSOVERLAP * 1.0 * (1-firstquarter) );    // Adjustable overlap over center position of joystick
  

  if ( stickPos >= midposition )                                        // UPPER half
    result = map(stickPos, midposition, MAXPOSITION, NORMALOUT, NORMALOUT);
    result += BOOST * tophalf;
    
  if ( stickPos < midposition )                                         // Upper part of LOWER half 
    result = map(stickPos, LOWPOSITION, midposition, LOWOUT , NORMALOUT);
    
  if (stickPos < LOWPOSITION)                                           // Lowest part of LOWER half
    result = map(stickPos, MINPOSITION, LOWPOSITION, MINOUT, LOWOUT);

  return (float)result/MAXOUT;
}



// ------------------------------------------------------------------------------
float cue (uint8_t channel, uint8_t opposite) {

  int result;
  int stickPos = avgarray[channel].getMedian();
  int oppPos = avgarray[opposite].getMedian();
  
//  float x = (stickPos-oppPos)/(float)MAXPOSITION;
//  float y = ((stickPos+oppPos)/(float)MAXPOSITION)-1;

  float tophalf      = map(constrain((stickPos+oppPos)/2, KNEEPOS*MAXPOSITION, MAXPOSITION), KNEEPOS*MAXPOSITION, MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  float firstquarter = map(constrain((stickPos+oppPos)/2, 0.5*MAXPOSITION, 0.75*MAXPOSITION), 0.5*MAXPOSITION, 0.75*MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  int midposition = MIDPOSITION-MIDPOSOVERLAP + ( MIDPOSOVERLAP * xfadecurve * (1-firstquarter) );    // Adjustable overlap over center position of joystick
  

  if ( stickPos >= midposition )                                        // UPPER half
    result = map(stickPos, midposition, MAXPOSITION, NORMALOUT, NORMALOUT);
    result += BOOST * tophalf;
    
  if ( stickPos < midposition )                                         // Upper part of LOWER half 
    result = map(stickPos, LOWPOSITION, midposition, LOWOUT , NORMALOUT);
    
  if (stickPos < LOWPOSITION)                                           // Lowest part of LOWER half
    result = map(stickPos, MINPOSITION, LOWPOSITION, MINOUT, LOWOUT);

  return (float)result/MAXOUT;
}











// ------------------------------------------------------------------------------
