#include <Arduino.h>
#include <muses.h>
#include <rotaries.h>
#include <2Dxfader.h>



// Volume
#define MAXVOL 447
#define MINVOL 0
#define STARTUPVOLUME 200
#define MASTERVOLUMESTEP 4   // 0,25dB * 4  per step on master volume knob
uint16_t one2mix = STARTUPVOLUME;
uint16_t two2mix = STARTUPVOLUME;
uint16_t one2cue = STARTUPVOLUME;
uint16_t two2cue = STARTUPVOLUME;



// ------------------------------------------------------------------------------------
void setup() {
  Serial.begin(115200);
  delay(200);
  Serial.println("Booting...");
  rotariesBegin();
  musesBegin();
}


void loop() {  
  
  readrotaries();  
  readfaders();
  setallvolumes();
  
  unsigned long loopnow = millis();
  static unsigned long timerone;
  if (loopnow - timerone >= 1000) {
    timerone = loopnow;
    
  }
}



// ------------------------------------------------------------------------------------
void readrotaries(void) {
  int16_t moved;

  // Gain INPUT ONE
  if(moved = rotaryPosition(rotONE)) {                      
    one2mix = constrain(one2mix+moved, MINVOL, MAXVOL); 
    one2cue = constrain(one2cue+moved, MINVOL, MAXVOL); 
  }
  
  // Gain INPUT TWO
  if(moved = rotTWO->getPosition()) {                       
    two2mix = constrain(two2mix+moved, MINVOL, MAXVOL); 
    two2cue = constrain(two2cue+moved, MINVOL, MAXVOL); 
  }
  
  // Volume MIX OUTPUT 
  if(moved = rotMIX->getPosition()) {                       
    if (one2mix) one2mix = constrain(one2mix+(moved*MASTERVOLUMESTEP), MINVOL, MAXVOL); 
    if (two2mix) two2mix = constrain(two2mix+(moved*MASTERVOLUMESTEP), MINVOL, MAXVOL); 
  }
  
  // Volume CUE OUTPUT
  if(moved = rotCUE->getPosition()) {                       
    if (one2cue) one2cue = constrain(one2cue+(moved*MASTERVOLUMESTEP), MINVOL, MAXVOL); 
    if (two2cue) two2cue = constrain(two2cue+(moved*MASTERVOLUMESTEP), MINVOL, MAXVOL); 
  }
}




// ------------------------------------------------------------------------------------
void setallvolumes(void) {

  // SOURCE ONE TO MIX
  uint16_t one2mixleft  = (uint16_t) (xfade(SRC1_LEFT, SRC2_RGHT) * (float)one2mix);
  uint16_t one2mixright = (uint16_t) (xfade(SRC1_RGHT, SRC2_LEFT) * (float)one2mix);
  volume(ONE2MIX, one2mixleft, one2mixright);
//  Serial.printf("%d \t %d \n", one2mixleft, one2mixright);
  
  // SOURCE TWO TO MIX
  uint16_t two2mixleft  = (uint16_t) (xfade(SRC2_LEFT, SRC1_RGHT) * (float)one2mix);
  uint16_t two2mixright = (uint16_t) (xfade(SRC2_RGHT, SRC1_LEFT) * (float)one2mix);
  volume(TWO2MIX, two2mixleft, two2mixright);

  // SOURCE ONE TO CUE
  uint16_t one2cueleft  = (uint16_t) ((cue(SRC2_LEFT, SRC1_RGHT)) * (float)one2mix);
  uint16_t one2cueright = (uint16_t) ((cue(SRC2_RGHT, SRC1_LEFT)) * (float)one2mix);
  volume(ONE2CUE, one2cueleft, one2cueright);

  // SOURCE TWO TO CUE
  uint16_t two2cueleft  = (uint16_t) ((cue(SRC1_LEFT, SRC2_RGHT)) * (float)one2mix);
  uint16_t two2cueright = (uint16_t) ((cue(SRC1_RGHT, SRC2_LEFT)) * (float)one2mix);
  volume(TWO2CUE, two2cueleft, two2cueright);
}




// ------------------------------------------------------------------------------------
